#include "bsp.h"
#include "mrfi.h"
#include "nwk_types.h"
#include "nwk_api.h"
#include "bsp_leds.h"
#include "bsp_buttons.h"
#include "vlo_rand.h"

void linkTo(void);
void MCU_Init(void);
void mrfiSpiInit(void);
__no_init volatile int tempOffset @ 0x10F4; // Temperature offset set at production
__no_init volatile char Flash_Addr[4] @ 0x10F0; // Flash address set randomly

void createRandomAddress(); 
static uint8_t sCB(linkID_t);
static uint8_t sPeerFrameSem;
bspIState_t intState;
unsigned char state; //vrsta senzora

linkID_t linkID1;
addr_t lAddr;
 
void main (void)
   {

   WDTCTL = WDTPW + WDTHOLD;                 // Stop WDT
      { 
      volatile int i;
      for(i = 0; i < 0xFFFF; i++){}
      }
   if( CALBC1_8MHZ == 0xFF )                 // Do not run if cal values are erased
      {
      volatile int i;
      P1DIR |= 0x03;
      BSP_TURN_ON_LED1();
      BSP_TURN_OFF_LED2();
      while(1)
         {
         for(i = 0; i < 0x5FFF; i++){}
         BSP_TOGGLE_LED2();
         BSP_TOGGLE_LED1();
      }  }
  
   // SimpliciTI will change port pin settings as well
   P1DIR = 0xFF;   P1OUT = 0x00;   
   P2DIR = 0x27;   P2OUT = 0x00;   
   P3DIR = 0xC0;   P3OUT = 0x00;   
   P4DIR = 0xFF;   P4OUT = 0x00;
  
   BSP_Init();
  
   if( Flash_Addr[0] == 0xFF && Flash_Addr[1] == 0xFF && Flash_Addr[2] == 0xFF && Flash_Addr[3] == 0xFF )
      {
      createRandomAddress();                  // set Random device address at initial startup
      }
   lAddr.addr[0]=Flash_Addr[0];   lAddr.addr[1]=Flash_Addr[1];   lAddr.addr[2]=Flash_Addr[2];   lAddr.addr[3]=Flash_Addr[3];
   SMPL_Ioctl(IOCTL_OBJ_ADDR, IOCTL_ACT_SET, &lAddr);
   BCSCTL1 = CALBC1_8MHZ;                    // Set DCO after random function
   DCOCTL = CALDCO_8MHZ;
   
   
   
   BCSCTL3 |= LFXT1S_2;                      // LFXT1 = VLO
   TACCTL0 = CCIE;                           // TACCR0 interrupt enabled
   TACCR0 = 12000;                           // ~ 1 sec
   TACTL = TASSEL_1 + MC_1;                  // ACLK, upmode
  
   // keep trying to join until successful. toggle LEDS to indicate that
   // joining has not occurred. LED3 is red but labeled LED 4 on the EXP
   // board silkscreen. LED1 is green.
   while (SMPL_NO_JOIN == SMPL_Init(sCB) )
   //while (SMPL_NO_JOIN == SMPL_Init((uint8_t (*)(linkID_t))0))
      {
      BSP_TOGGLE_LED1();
      BSP_TOGGLE_LED2();;
      __bis_SR_register(LPM3_bits + GIE);     // LPM3 with interrupts enabled
      }
   // unconditional link to AP which is listening due to successful join.
   
   linkTo();
   }

void createRandomAddress()
   {
   unsigned int rand, rand2;
   do
      {
      rand = TI_getRandomIntegerFromVLO();    // first byte can not be 0x00 of 0xFF
      } while( (rand & 0xFF00)==0xFF00 || (rand & 0xFF00)==0x0000 );
   rand2 = TI_getRandomIntegerFromVLO();
  
   BCSCTL1 = CALBC1_1MHZ;                    // Set DCO to 1MHz
   DCOCTL = CALDCO_1MHZ;
   FCTL2 = FWKEY + FSSEL0 + FN1;             // MCLK/3 for Flash Timing Generator
   FCTL3 = FWKEY + LOCKA;                    // Clear LOCK & LOCKA bits
   FCTL1 = FWKEY + WRT;                      // Set WRT bit for write operation

   Flash_Addr[0]=(rand>>8) & 0xFF;
   Flash_Addr[1]=rand & 0xFF;
   Flash_Addr[2]=(rand2>>8) & 0xFF; 
   Flash_Addr[3]=rand2 & 0xFF; 
  
   FCTL1 = FWKEY;                            // Clear WRT bit
   FCTL3 = FWKEY + LOCKA + LOCK;             // Set LOCK & LOCKA bit
   }

void linkTo()
   {
   //uint8_t  msg[3];
   // keep trying to link...
   while (SMPL_SUCCESS != SMPL_Link(&linkID1))
      {
      __bis_SR_register(LPM3_bits + GIE);     // LPM3 with interrupts enabled
      BSP_TOGGLE_LED1();
      BSP_TOGGLE_LED2();
      }
   
   // Turn off all LEDs
   if (BSP_LED1_IS_ON())
      {
      BSP_TOGGLE_LED1();
      }
   if (BSP_LED2_IS_ON())
      {
      BSP_TOGGLE_LED2();
      }
  
   //mrfiSpiInit();
   MRFI_SetRxAddrFilter(lAddr.addr);
   MRFI_EnableRxAddrFilter();
   
   while (1)
      {
      if (sPeerFrameSem)
         {
         uint8_t msg[2],len;
         msg[0]=120;
         SMPL_Receive(linkID1, msg, &len);
         //if (SMPL_SUCCESS==SMPL_Receive(linkID1, msg, &len))
            {
            switch (msg[0])
               {
               case 0x00://resetuj konfiguraciju
                  {
                  state=0;
                  break;
                  }
               case 0x01://konfiguracija da radi kao digitalni izlaz
                  {
                  state=1; //podesavanje diode
                  //  //if (SMPL_SUCCESS == SMPL_Send(linkID1, "OK", 2));
                  break;
                  }
               case 0x02: //konfiguracija da radi kao digitalni uzlaz, koristi se taster
                  {
                  state=2;  
                  P1DIR &= ~0x04;
                  P1REN |=  0x04;//pull up/down enabled
                  P1OUT |=  0x04;//pull high
                  break;
               }
            case 0x03: //meri temperaturu
               {
               state = 3;
               break;
               }
            case 0x04://ugasi diodu
               {
               if (state==1)  
                  if (BSP_LED1_IS_ON())
                     {
                     BSP_TOGGLE_LED1();
                     }   
               break;
               }
            case 0x05://upali diodu
               {
               if (state==1)
                  if (!(BSP_LED1_IS_ON()))
                     {
                     BSP_TOGGLE_LED1();
                     }                                       
               break;
                  }  
               case 0x06: //daj mi temperaturu?
                  {
                  if (state==3)
                     {
                     }
                  break;  
                  }
               case 0x07: //daj mi taster
                  {
                  if (state==2)
                     {
                     BSP_TURN_ON_LED2();
                     if (P1IN&0x04) msg[0]='A';
                     else msg[0]='B';
                     SMPL_Send(linkID1, msg, 1);
                     }
                  break;
                  }
               case 0xFF: //daj mi sta si
                  {
                  SMPL_Send(linkID1, &state, 1);
                  break;
                  }
               default: break;
            }  }
         BSP_ENTER_CRITICAL_SECTION(intState);
         sPeerFrameSem--;
         BSP_EXIT_CRITICAL_SECTION(intState);
         }      
      }  
   }

/*------------------------------------------------------------------------------
* ADC10 interrupt service routine
------------------------------------------------------------------------------*/
#pragma vector=ADC10_VECTOR
__interrupt void ADC10_ISR(void)
   {
   __bic_SR_register_on_exit(CPUOFF);        // Clear CPUOFF bit from 0(SR)
   }

/*------------------------------------------------------------------------------
* Timer A0 interrupt service routine
------------------------------------------------------------------------------*/
#pragma vector=TIMERA0_VECTOR
__interrupt void Timer_A (void)
   {
   __bic_SR_register_on_exit(LPM3_bits);        // Clear LPM3 bit from 0(SR)
   }

/*------------------------------------------------------------------------------
* Runs in ISR context. Reading the frame should be done in the
* application thread not in the ISR thread.
------------------------------------------------------------------------------*/
static uint8_t sCB(linkID_t lid)
   {   
   if (lid)
      {
      sPeerFrameSem++;
      }
   return 0;
   }
